#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import os, sys, tempfile
import gtk, gobject
import gtk.glade
from creole.cfgparser import *
from creole.error import TypeEoleError
from creole import config
from creole.fonctionseole import get_module_name
import optparse
registered = 0
try:
    from zephir.zephir_conf.zephir_conf import adresse_zephir, id_serveur
    from zephir.lib_zephir import xmlrpclib
    registered = 1
except:
    import xmlrpclib
    adresse_zephir = None
    id_serveur = -1

glade_file = os.path.join(config.datadir, 'gen_config.glade')

class ConfigGui:
    """
    Classe interface graphique pour configuration
    """

    def __init__(self, interface_file, dir_config=['/etc/eole/dicos/', '/etc/eole/dicos/local/', '/etc/eole/dicos/variante/'], mode='normal', options=None, filename=None):
        self.dir_config = dir_config
        # parsage du fichier de theme
        gtk.rc_parse(os.path.join(config.datadir, 'eole.rc'))
        self.interface_file = interface_file
        self.glade = gtk.glade.XML(interface_file,'main')
        self.glade_select = None
        self.window = self.glade.get_widget('main')
        # mode par défaut
        self.mode = mode
        # proxy xml-rpc vers Zephir
        self.zephir = None
        self.addr_zephir = None
        self.login_zephir = None
        # n° du serveur zephir en cours de traitement
        self.zephir_server = None
        self.zephir_modules = {}
        self.zephir_rne = None
        self.dial_select = None
        # variable cachées / grisées
        self.hide_vars = True
        # module du serveur local
        self.module_import = None
        #table contenant les edits + labels a droite
        self.table = self.glade.get_widget('table')
        #image de titre
        self.image = self.glade.get_widget('titre')
        #vbox de boutons des noms de familles
        self.vbox_button = self.glade.get_widget('vbox_button')
    
        sig = {'on_main_delete_event' : self._callback_button_quit,
               'on_quitter_activate' : self._callback_button_quit,
               'on_button_valid_group_clicked' : self._save_values,
               'on_button_load_default_clicked' : self._callback_load_defaults,
               'on_charger_defauts_activate' : self._callback_button_defaults,
               'on_enregistrer_activate' : self._callback_button_save,
               'on_ouvrir_activate' : self._callback_button_load,
               'on_import_creole1_activate' : self._callback_button_import,
               'on_normal_activate' : self._change_mode,
               'on_hide_vars_activate' : self._change_affvar,
               'on_zephir_get_activate' : self.zephir_get,
               'on_zephir_send_activate' : self.zephir_send,
               'on_zephir_connect_activate' : self.zephir_connect,
               'on_zephir_disconnect_activate' : self.zephir_disconnect,
               'on_zephir_select_activate' : self.zephir_select,
               'on_importcreole1_activate' : self.zephir_import
              }
        if options and options.window_mode:
            self.window.resize(800, 600)
        else:  # For use with framebuffer:
            self.window.fullscreen()
            self.window.maximize()
            screen = self.window.get_screen()
            self.window.set_size_request(screen.get_width(), screen.get_height())
        self.glade.get_widget('zephir_send').set_sensitive(False)
        self.glade.get_widget('zephir_get').set_sensitive(False)
        self.glade.get_widget('zephir_select').set_sensitive(False)
        self.glade.get_widget('zephir_disconnect').set_sensitive(False)
        self.glade.get_widget('zephir_import').set_sensitive(False)
        self.glade.signal_autoconnect(sig)
        self.init_default_dict()
        # dictionnaire par defaut
        if os.path.isfile('/etc/eole/config.eol'):
            self.filename = '/etc/eole/config.eol'
        else:
            self.filename = '/root/zephir.eol'
        if filename != None and os.path.isfile(filename):
            if self.eole_dico.load_values(filename) != False:
                self.filename = filename
                self.valid_families = []
                self.main()
                self._refresh_display()
        # si enregistré sur zephir : passage en mode distant si demandé
        elif registered:
            self.zephir_connect()

    def init_default_dict(self):
        #pour stocker les edits et les labels
        self.stock = {}
        #pour stocker les boutons de familles
        self.button_stock = {}
        # dictionnaire des variables en erreur
        self.var_status = {}
        #flag pour savoir si on a sauve avant de quitter
        self.save_flag = True
        self.valid_families = []
        #variable indiquant la famille en cours d'édition
        self.edited_family = ""
        #si une case a été éditée (éviter la confirmation à chaque chgmt de famille
        self.modified_flag = False
        self.eole_dico = EoleDict(templatedir=config.templatedir)
        # chargement des dictionnaires
        self.eole_dico.read_dir(self.dir_config)
        mod_libelle = 'Eole'
        if os.path.isfile('/etc/eole/version'):
            f_version = open('/etc/eole/version')
            version = f_version.read().split('\n')[0]
            f_version.close()
            mod_libelle = version.capitalize().split('-')[0]
            self.set_title(mod_libelle)

    def zephir_connect(self, *args):
        # saisie login/mdp
        glade_zeph = gtk.glade.XML(self.interface_file,'dialog_zeph')
        dial_zeph = glade_zeph.get_widget('dialog_zeph')
        focus = 'pwd_zeph'
        if self.login_zephir is not None:
            glade_zeph.get_widget('login_zeph').set_text(self.login_zephir)
        else:
            focus = 'login_zeph'
        if self.addr_zephir is not None: 
            glade_zeph.get_widget('adresse_zeph').set_text(self.addr_zephir)
        elif adresse_zephir is not None:
            glade_zeph.get_widget('adresse_zeph').set_text(adresse_zephir)
        else:
            focus = 'adresse_zeph'
        glade_zeph.get_widget(focus).grab_focus()
        response = dial_zeph.run()
        if response == gtk.RESPONSE_OK:
            # récupération login/mot de passe
            login = glade_zeph.get_widget('login_zeph').get_text()
            passwd = glade_zeph.get_widget('pwd_zeph').get_text()
            addr =  glade_zeph.get_widget('adresse_zeph').get_text()
            self.addr_zephir = addr
            self.login_zephir = login
            zephir = xmlrpclib.ServerProxy('https://%s:%s@%s:7080' % (login, passwd, addr))
            try:
                data = zephir.modules.get_module()
                assert data[0] == 1
                for mod in data[1]:
                    # on ne gère que les modules Eole2 ?
                    if mod['version'] >= 2:
                        self.zephir_modules[mod['id']] = (mod['libelle'], True)
                    else:
                        self.zephir_modules[mod['id']] = (mod['libelle'], False)
                self.zephir = zephir
                self.glade.get_widget('zephir_send').set_sensitive(True)
                self.glade.get_widget('zephir_get').set_sensitive(True)
                self.glade.get_widget('zephir_select').set_sensitive(True)
                self.glade.get_widget('zephir_disconnect').set_sensitive(True)
                self.glade.get_widget('zephir_import').set_sensitive(True)
                # chargement de la liste des établissements disponibles
                code, etabs = self.zephir.etabs.get_etab()
                if code != 1:
                    self._error_window(unicode('Erreur de recherche des établissements sur zephir : %s' % etabs, config.charset))
                    self.etabs = []
                else:
                    self.etabs = [etab['rne'] for etab in etabs]
                    self.etabs.sort()
                if id_serveur != -1 and self.zephir_server is None:
                    self.zephir_server = id_serveur
                    self.zephir_initialize()
            except:
                self._error_window('Echec de la connexion à Zephir')
        dial_zeph.destroy()

    def zephir_disconnect(self, *args):
        self.glade.get_widget('zephir_send').set_sensitive(False)
        self.glade.get_widget('zephir_get').set_sensitive(False)
        self.glade.get_widget('zephir_select').set_sensitive(False)
        self.glade.get_widget('zephir_disconnect').set_sensitive(False)
        self.glade.get_widget('zephir_import').set_sensitive(False)
        self.zephir = None
        self.zephir_rne = None
        self.zephir_server = None
        self.init_default_dict()
        self.main()
        self._refresh_display()

    def set_title(self, libelle, size=42):
        # met à jour le libellé au dessus de la zone de saisie
        label = self.glade.get_widget('label_title')
        label.set_markup('<span foreground="#a0569a" font_desc="monospace bold %s">%s</span>' % (str(size), libelle))

    def build_led(self, green = False):
        """leds vertes et rouges pour la validation des onglets"""
        led = gtk.Image()
        if green:
            led.set_from_stock(gtk.STOCK_YES, gtk.ICON_SIZE_BUTTON)
        else:
            led.set_from_stock(gtk.STOCK_NO, gtk.ICON_SIZE_BUTTON)
        return led

    def _quitter(self):
        """
        quitte l'application principale
        """
        gtk.main_quit()

    def _callback_load_defaults(self, *args):
        """
        charge dans les edits les valeurs par defaut pour la famille editée
        """
        if self.edited_family != '':
            text = "Voulez-vous vraiment charger les valeurs par defaut pour ce groupe?"
            reponse = self._question_window(text)
            if reponse == 1:
                for key in self.eole_dico.families[self.edited_family]['vars']:
                    self.eole_dico.variables[key].val = self.eole_dico.variables[key].get_value(default=True)
                # deuxième passe pour les valeurs calculées
                for key in self.eole_dico.families[self.edited_family]['vars']:
                    self.eole_dico.variables[key].val = self.eole_dico.variables[key].get_value(default=True)
                    try:
                        self.eole_dico.check_conditions(key, force=True)
                    except Exception, e:
                        self._error_window(str(e))
                if self.edited_family in self.valid_families:
                    self.valid_families.remove(self.edited_family)
                self._update_buttons()
                self._clear_table()
                self._build_table(self.edited_family)
                self._update_display()
     
    def _callback_button_quit(self, *args):
        """
        click sur bouton quitter
        """
        #on a pas sauve avant de quitter
        if self.save_flag == False:
            widgets = gtk.glade.XML(self.interface_file,'dialog_confirm')
            dial_conf = widgets.get_widget('dialog_confirm')
            dial_label = widgets.get_widget('label_confirm')
            text = "Voulez vous sauver le fichier avant de quitter ?"
            dial_label.set_text(text)
            dial_conf.set_title("Confirmation de sortie")
            reponse = dial_conf.run()
            dial_conf.destroy()
            if reponse == gtk.RESPONSE_CANCEL:
                return True
            elif reponse == gtk.RESPONSE_YES:
                # sauvegarde du fichier
                self._callback_button_save(None)
        self._quitter()
   
    def set_filters(self, filedialog):
        current_dir = os.path.dirname(self.filename)
        filedialog.set_current_folder(current_dir)
        # nb : le raccourci sur le home_dir est créé automatiquement
        filedialog.add_shortcut_folder('/etc/eole')
        filter = gtk.FileFilter()
        filter.set_name("Tous les fichiers")
        filter.add_pattern("*")
        filedialog.add_filter(filter)
        filter = gtk.FileFilter()
        filter.set_name("Fichiers de valeurs Creole")
        filter.add_pattern("*.eol")
        filedialog.add_filter(filter)
        filedialog.set_filter(filter)

    def _callback_button_save(self, widget):
        """
        click sur bouton sauver
        """
        # avant tout, on sauvegarde
        if not self._save_values(None):
            return False
        ok = True
        boutons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE, gtk.RESPONSE_ACCEPT)
        save_dlg = gtk.FileChooserDialog(action = gtk.FILE_CHOOSER_ACTION_SAVE, buttons = boutons)
        save_dlg.set_current_name(unicode(self.filename, 'UTF-8'))
        # on demande validation si le fichier existe
        save_dlg.set_do_overwrite_confirmation(True)
        self.set_filters(save_dlg)

        if save_dlg.run() == gtk.RESPONSE_ACCEPT:
            ok = True
            text = "Enregistrement dans le fichier selectionne impossible"
            try:
                out_file = save_dlg.get_filename()
                #erreurs dans les groupes, on restaure les valeurs précédentes
                if self._check_groups() == False:
                    save_dlg.destroy()
                    return False
                ok = self.eole_dico.save_values(out_file)
            except TypeEoleError, e:
                ok = False
                text = e.args[0]
            else:
                self.valid_families = self.eole_dico.family_order
                # update des onglets (led verte)
                for tab in self.button_stock.items():
                    tab[1].set_image(self.build_led(True))
                self.save_flag = True
            if ok == False:
                self._error_window(text)
            else:
                # si mode zephir, on propose la sauvegarde à distance
                if self.zephir is not None and self.zephir_server is not None:
                    if self._question_window("Envoyer la modification sur zephir ?", "Sauvegarde"):
                        self.zephir_send()
        save_dlg.destroy()

    def _callback_button_load(self, widget):
        """
        click sur bouton ouvrir
        """
        ok = True
        boutons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_ACCEPT)
        open_dlg = gtk.FileChooserDialog(action = gtk.FILE_CHOOSER_ACTION_OPEN, buttons = boutons)
        open_dlg.select_filename(unicode(self.filename, 'UTF-8'))
        self.set_filters(open_dlg)
        text = "Chargement depuis le fichier selectionne impossible"
        if open_dlg.run() == gtk.RESPONSE_ACCEPT:
            try:
                if self.eole_dico.load_values(open_dlg.get_filename()) == False:
                    ok = False
                else:
                    self.filename = open_dlg.get_filename()
                    self.valid_families = []
                    self._refresh_display()
            except Exception, e:
                ok = False
                text = "Erreur au chargement des valeurs :\n" + e.__class__.__name__ + ' : ' + str(e)
        open_dlg.destroy()
        if ok == False:
            self._error_window(text)

    def _callback_button_import(self, widget):
        """
        click sur bouton import Creole1
        """
        boutons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_ACCEPT)
        open_dlg = gtk.FileChooserDialog(action = gtk.FILE_CHOOSER_ACTION_OPEN, buttons = boutons)
        open_dlg.select_filename(unicode(self.filename, 'UTF-8'))
        self.set_filters(open_dlg)
        errors = []
        warnings = []
        if open_dlg.run() == gtk.RESPONSE_ACCEPT:
            try:
                errors, warnings = self.eole_dico.import_values(open_dlg.get_filename())
            except ConfigError, e:
                open_dlg.destroy()
                self._error_window(str(e))
                return False
            # mise à jour de l'affichage
            self.valid_families = []
            self._refresh_display()
        open_dlg.destroy()
        msg = []
        if errors != []:
            msg.append(u"\nerreurs rencontrées lors de l'import :")
            msg.extend(errors)
        if warnings != []:
            msg.append(u"\ndonnées non validées :")
            msg.extend(warnings)
        if msg != []:
            self._error_window("\n".join(msg))
      
    def _callback_button_defaults(self, widget):
        """
        click sur bouton charger defauts
        """
        text = "Voulez-vous vraiment charger les valeurs par defaut?"
        reponse = self._question_window(text)
        if reponse == 1:
            #on met la valeur par defaut en interne
            self._restore_defaults()
            #on met a jour l'affichage
            self.valid_families = []
            self._refresh_display()

    def zephir_get(self, *args):
        # si mode zephir : on récupère les valeurs automatiquement
        ok = True
        if self.zephir is not None:
            try:
                if self.zephir_server is None:
                    # le n° du serveur a configurer n'est pas connu, on le demande :
                    self._error_window('Aucun serveur sélectionné')
                    return False
                code, res = self.zephir.serveurs.get_dico(self.zephir_server, 'modif_config')
                # chargement des valeurs
                values = eval(res[-1])
                self.eole_dico.store._sections = values
                self.eole_dico._load_values()
                text = "Erreur de chargement depuis zephir"
            except Exception, e:
                code = 0
                text = str(e)
            if code == 0:
                ok = False
            if ok == False:
                self._error_window(text)
            self.valid_families = []
            self._refresh_display()
        else:
            return False

    def zephir_send(self, widget=None, *args):
        if self.zephir is not None:
            # vérification des valeurs en cours d'édition
            if not self._save_values(None):
                return False
            if self.zephir_server is None:
                self._error_window('Aucun serveur sélectionné')
                return False
            else:
                ok = True
                try:
                    # si le fichier existe, on demande confirmation
                    ok = self.eole_dico.save_values('/dev/null')
                    # erreurs dans les groupes, on restaure les valeurs precedentes
                    if self._check_groups() == False:
                        return False
                except TypeEoleError, e:
                    ok = False
                    text = e.args[0]
                else:
                    self.valid_families = self.eole_dico.family_order
                    # update des onglets (led verte)
                    for tab in self.button_stock.items():
                        tab[1].set_image(self.build_led(True))
                    self.save_flag = True
                if ok == False:
                    self._error_window(text)
                else:
                    # configuration validée, envoi sur zephir
                    conf = unicode(str(self.eole_dico.store._sections), config.charset)
                    try:
                        code, msg = self.zephir.serveurs.save_conf(self.zephir_server, [conf])
                    except Exception, e:
                        code = 0
                        msg = str(e)
                    if code == 0:
                        text = msg
                        self._error_window(text)
                    else:
                        # si lancé depuis le bouton d'envoi, demande de sauvegarde locale ?
                        if widget is not None:
                            if self._question_window("Sauvegarder également en local ?", "Envoi Zephir"):
                                try:
                                    self.eole_dico.save_values('/etc/eole/config.eol')
                                except:
                                    self._error_window(unicode("Erreur de mise à jour de la configuration locale (/etc/eole/config.eol)", config.charset))
                        # sauvegarde réussie
                        MessageWindow(unicode("Envoi à Zephir", config.charset), unicode("configuration sauvegardée sur Zephir", config.charset), self.window).getrc()
        return False

    def zephir_select(self, *args):
        if self.module_import != None:
            if self.glade_select != None:
                # changement de mode (import/édition) : on reset le dialogue de sélection
                self.glade_select.get_widget('combo_rne').get_child().set_text('')
                tree = self.glade_select.get_widget('tree_etab')
                model = tree.get_model()
                model.clear()
            self.module_import = None
        self._zephir_select()

    def _zephir_select(self):
        if self.zephir is None:
            self.zephir_connect()
            if self.zephir is None:
                self._error_window(unicode("Vous n'êtes pas authentifié sur Zephir", config.charset))
                return False
        if self.dial_select is None:
            self.glade_select = gtk.glade.XML(self.interface_file,'dialog_select')
            self.dial_select = self.glade_select.get_widget('dialog_select')
            tree = self.glade_select.get_widget('tree_etab')
            tree.set_model(gtk.ListStore(gobject.TYPE_INT, gobject.TYPE_STRING, gobject.TYPE_STRING))
            tree.append_column(gtk.TreeViewColumn('id', gtk.CellRendererText(), text=0))
            tree.append_column(gtk.TreeViewColumn('module', gtk.CellRendererText(), text=1))
            tree.append_column(gtk.TreeViewColumn('libelle', gtk.CellRendererText(), text=2))
            # récupération des données accessibles sur zephir
            self._fill_dial_select()
        else:
            tree = self.glade_select.get_widget('tree_etab')
        resp = self.dial_select.run()
        if resp == gtk.RESPONSE_OK:
            # récupération du serveur sélectionné
            model, iter_sel = tree.get_selection().get_selected()
            if iter_sel != None:
                self.zephir_server = int(model.get_value(iter_sel, 0))
                if self.module_import == None:
                    # édition d'un serveur NG quelconque de zephir
                    self.zephir_initialize()
                else:
                    # import d'une configuration creole1
                    self.zephir_import_conf()
        self.dial_select.hide()

    def _fill_dial_select(self):
        # remplissage de la combobox
        combo = self.glade_select.get_widget('combo_rne')
        combo.set_model(gtk.ListStore(gobject.TYPE_STRING))
        combo.set_text_column(0)
        for etab in self.etabs:
            combo.append_text(etab)
        combo.get_child().connect('changed', self.update_rne)

    def update_rne(self, *args):
        combo = self.glade_select.get_widget('combo_rne')
        rne = combo.get_active_text()
        if rne in self.etabs:
            self.zephir_rne = rne
            code, serveurs = self.zephir.serveurs.groupe_serveur({'rne':rne})
            if code != 1:
                self._error_window(unicode('Erreur de lecture des informations sur zephir : %s' % serveurs, config.charset))
            else:
                tree = self.glade_select.get_widget('tree_etab')
                model = tree.get_model()
                model.clear()
                for serveur in serveurs:
                    serveur_ok = False
                    if self.module_import == None:
                        # mode normal, on n'affiche que les serveurs eole2
                        if self.zephir_modules[int(serveur['module_actuel'])][1] == True:
                            serveur_ok = True
                            
                    else:
                        # si on est en mode import, on ne prend que les modules eole1 du même type
                        if self.zephir_modules[int(serveur['module_actuel'])][0].startswith(self.module_import):
                            if self.zephir_modules[int(serveur['module_actuel'])][1] == False:
                                serveur_ok = True
                    if serveur_ok == True:
                        model.append([serveur['id'], self.zephir_modules[int(serveur['module_actuel'])][0], serveur['libelle']])

    def zephir_import(self, *args):
        """fonction d'import des données d'un serveur creole1
        """
        if self.module_import == None:
            self.init_default_dict()
            self.main()
            self._refresh_display()
            # changement de mode (import/édition) : on reset le dialogue de sélection
            if self.glade_select != None:
                self.glade_select.get_widget('combo_rne').get_child().set_text('')
                tree = self.glade_select.get_widget('tree_etab')
                model = tree.get_model()
                model.clear()
            self.module_import = get_module_name().split('-')[0]
        self._zephir_select()

    def zephir_initialize(self):
        try:
            module_ok = self.zephir_title()
            if module_ok == True:
                code, res = self.zephir.serveurs.get_dico(self.zephir_server, 'config')
                if self.zephir_server != id_serveur:
                    self.eole_dico = EoleDict(templatedir=config.templatedir)
                    # si on édite un autre serveur, on prend les dictionnaires présents sur zephir
                    dicos = [xmlrpclib.base64.decodestring(dico_zeph) for dico_zeph in res[:-1]]
                    # chargement de tous les dictionnaires disponibles
                    for xml_dict in dicos:
                        self.eole_dico.read_string(xml_dict)
                self.modified_flag = False
                self.edited_family = ''
                #construit et affiche les boutons a gauche
                self._build_buttons()
                # rafraichit l'affichage
                self.valid_families = []
                for fam in self.eole_dico.family_order:
                    if not self.eole_dico.families[fam]['hidden'] == True:
                        if self.mode == 'normal' and fam not in self.eole_dico.normal_families:
                            continue
                        self.button_stock[fam].set_active(True)
                        break
                self.zephir_get()
                self._refresh_display()
        except Exception, e:
            self._error_window('Erreur de lecture des informations sur zephir \n(%s)' % str(e))
        return False

    def zephir_import_conf(self):
        """
        import d'une configuration creole1 depuis zephir
        """
        # récupération du dictionnaire de valeurs creole1 sur zephir
        module_ok = self.zephir_title()
        if module_ok == True:
            code, res = self.zephir.serveurs.get_dico(self.zephir_server, 'modif_config')
            # fichier temporaire de conversion
            descr, temp_conffile = tempfile.mkstemp(suffix=".eol")
            f_tmp = open(temp_conffile, 'w')
            for dic in res:
                f_tmp.write(xmlrpclib.base64.decodestring(dic))
            f_tmp.close()
            try:
                errors, warnings = self.eole_dico.import_values(temp_conffile)
            except ConfigError, e:
                open_dlg.destroy()
                self._error_window(str(e))
                return False
            os.unlink(temp_conffile)
            # mise à jour de l'affichage
            self.dial_select.hide()
            self.valid_families = []
            self._refresh_display()
            msg = []
            if errors != []:
                msg.append(u"\nerreurs rencontrées lors de l'import :")
                msg.extend(errors)
            if warnings != []:
                msg.append(u"\ndonnées non validées :")
                msg.extend(warnings)
            if msg != []:
                self._error_window("\n".join(msg))

    def zephir_title(self, *args):
        # infos sur le serveur
        try:
            code, info_serv = self.zephir.serveurs.get_serveur(self.zephir_server)
        except xmlrpclib.ProtocolError:
            self._error_window('Accès au serveur %s non autorisé' % self.zephir_server)
        except Exception, e:
            self._error_window('Erreur de lecture des informations sur zephir \n(%s)' % str(e))
        else:
            libelle = info_serv[0]['libelle']
            id_mod = info_serv[0]['module_actuel']
            if id_mod in self.zephir_modules.keys():
                if self.module_import is None:
                    if self.zephir_modules[id_mod][1] == True:
                        server_title = """%s - %s (%s)\n %s""" % (info_serv[0]['rne'], info_serv[0]['libelle'], self.zephir_server, self.zephir_modules[id_mod][0])
                        self.set_title(server_title, size=20)
                        return True
                else:
                    if self.zephir_modules[id_mod][1] == False:
                        return True
                self._error_window('Module du serveur %s non géré' % self.zephir_server)
        return False

    def _refresh_display(self, * args):
        if self.edited_family == '':
            self.edited_family = self.eole_dico.family_order[0]
            self._untoggle_buttons(self.edited_family)

        self._clear_table()
        self._build_table(self.edited_family)
        #on met a jour l'affichage
        self._update_display()
        self.save_flag = True
        self._update_buttons()
        #active/desactive boutons de famille selon champ hidden
        self._disable_buttons()
        #active/desactive les labels/edits selon leur champ hidden
        self._disable_edits()
        self._update_display()

    def _callback_combobox_changed(self, widget, name):
        """
        callback quand on change la valeur d'une combobox
        """
        if type(widget) == gtk.ComboBoxEntry:
            # les comboboxentry sont traités de la mm façon que les entry
            return False
        val = widget.get_active_text()
        try:
            self.eole_dico.set_value(name, val)
            self.var_status[name] = ""
        except Exception, e:
            self._error_window(str(e.args[0]))
            return False
        #active/desactive boutons de famille selon champ hidden
        self._disable_buttons()
        #active/desactive les labels/edits selon leur champ hidden
        self._disable_edits()
        self.modified_flag = True
        #self._update_display()
 
    def _callback_modified_textbox(self, *args):
        """
        modification d'une case
        """
        if self.edited_family in self.valid_families:
            self.valid_families.remove(self.edited_family)
        self.button_stock[self.edited_family].set_image(self.build_led())
        self.modified_flag = True

    def _update_display(self):
        """
        Met a jour l'affichage des edits selon les valeurs courantes
        """
        errors = []
        for i in self.stock.keys():
            cpt_saisie = 0
            for saisie in self.stock[i][0].get_children():
                text = self._get_saisie_text(i)
                if text == []:
                    # sauvegarde du fichier
                    text = ['']
                if type(saisie) == gtk.Entry:
                    # widget Entry
                    saisie.set_text(text[cpt_saisie])
                    cpt_saisie += 1
                elif type(saisie) == gtk.ComboBoxEntry:
                    # widget ComboBoxEntry
                    saisie.get_child().set_text(text[cpt_saisie])
                    cpt_saisie += 1
                else:
                    # widget ComboBox
                    model = saisie.get_model()
                    cpt = 0
                    while cpt < len(model):
                        if model[cpt][0] == text[cpt_saisie]:
                            break
                        cpt += 1
                    if cpt == len(model):
                        errors.append(str(i))
                    saisie.set_active(cpt)
                    cpt_saisie += 1
        if errors != []:
            text = "Variable(s) inconnue : \n%s" % "\n".join(errors)
            self._error_window(text)
                    
                
    def _restore_defaults(self):
        """
        charge les valeurs par defaut dans les valeurs courantes
        """
        for fam in self.eole_dico.family_order:
            for var in self.eole_dico.families[fam]['vars']:
                self.eole_dico.variables[var].val = self.eole_dico.variables[var].get_value(default=True)
            # deuxième passe (valeurs calculées)
            for var in self.eole_dico.families[fam]['vars']:
                self.eole_dico.variables[var].val = self.eole_dico.variables[var].get_value(default=True)
                self.eole_dico.check_conditions(var, force=True)
  
    def _update_buttons(self):
        """updates led on all buttons"""
        for cle, button in self.button_stock.items():
            if cle in self.valid_families:
                button.set_image(self.build_led(True))
            else:
                button.set_image(self.build_led())

    def _build_buttons(self):
        """
        construit les boutons des familles a gauche et les stocke dans un dictionnaire
        """
        # réninitialisation pour le changement de mode
        for child in self.vbox_button.get_children():
            self.vbox_button.remove(child)
        
        # on affiche les onglets, dans l'ordre de  eole_dico.family_order, les familles expert en dernier
        button_order =  []
        for fam in self.eole_dico.family_order:
            if fam in self.eole_dico.normal_families:
                button_order.append(fam)
        for fam in self.eole_dico.family_order:
            if fam not in self.eole_dico.normal_families:
                button_order.append(fam)
        for cle in button_order:
            if self.mode == 'normal' and cle not in self.eole_dico.normal_families:
                continue
            button = gtk.ToggleButton(cle.capitalize(), None)
            button.set_alignment(0, -1)
            button.show()
            self.button_stock[cle] = button
            self.vbox_button.pack_start(button)
            #connecte le callback pour quand on clique sur le bouton
            button.connect("toggled", self._callback_toggle_button, cle)
            #ajoute l'aide sur les boutons
            self._set_button_tooltip(cle, button)
        # voyants pour la validation des données
        self._update_buttons()
        self.vbox_button.show_all()

    def _callback_toggle_button(self, widget, name):
        """
        callback lors de l'appui sur un bouton de famille a gauche
        On va vider puis construire la table a droite selon les variables de la famille
        @param widget: widget qui declenche le callback
        @param name: nom de la famille sur laquelle on clique
        """
        if widget.get_active() == True:
            # éviter le 'first time'
            if self.edited_family != '' and self.modified_flag == True:
                reponse = self._question_window("Valider les modifications ?")
                if reponse == 1:
                    if not self._save_values(None):
                        self._untoggle_buttons(self.edited_family)
                        return False

            self.modified_flag = False
            #on 'reset' la scrollbar de la fenêtre principale
            scrollwin = self.glade.get_widget('scrolledwindow1')
            scrollwin.do_scroll_child(scrollwin, gtk.SCROLL_START, False)
            #detruit les widgets contenus dans la table
            self._clear_table()
            #reconstruit la table selon la famille en cours
            self._build_table(name)
            #ne met actif que le bouton sur lequel on a appuye
            self._untoggle_buttons(name)
            #on stocke le nom de la famille éditée
            self.edited_family = name
    
    def _get_saisie_text(self, var, default=False, calculated=False):
        """
        renvoi la chaine de valeur associee a var (au lieu d'une liste, comme ca on peut afficher)
        """
        if calculated == False:
            val_actu = self.eole_dico.variables[var].get_value(default)
        else:
            val_actu = self.eole_dico.variables[var].get_calculated_value()
       
        return val_actu
    
    def _callback_focus_in_edit(self, widget, event, var, force=False):
        """
        focus rentre dans edit
        """
        if len(self.eole_dico.variables[var].fills) > 0:
            if widget.get_text() == '' or force == True:
                #sauvegarde 
                sauv_var = {}
                self._push_vals(sauv_var)
                cpt = 0
                #pour trouver l'index de l'edit dans la liste des edits de chaque variable
                for i in self.stock[var][0].get_children():
                    if i == widget:
                        break
                    cpt += 1
                    
                for key in self.stock.keys():
                    tmp = []
                    try:
                        for saisie in self.stock[key][0].get_children():
                            chaine = saisie.get_text()
                            chaine.strip()
                            tmp.append(chaine)
                        self.eole_dico.variables[key].val = tmp
                        self.eole_dico.variables[key].valdefault = ['']
                        self.eole_dico.variables[key].valeole = ['']
                    except Exception, e:
                        pass
                try:
                    text = self._get_saisie_text(var, calculated=True)
                except:
                    text = []
                if text != []:
                    if len(text) == 1:
                        val = text[0]
                    else:
                        val = text[cpt]
                else:
                    val = ''
                widget.set_text(val)
                #restauration
                self._pop_vals(sauv_var)
   
    def _set_label_tooltip(self, variable, label, edit):
        """
        Met un tooltip d'aide sur un label + edit
        """
        if self.eole_dico.helps['variables'].has_key(variable):
            text = self.eole_dico.helps['variables'][variable]
            tip = gtk.Tooltips()
            tip.set_tip(label, text)
            tip.set_tip(edit, text)
            tip.enable()
    
    def _set_button_tooltip(self, family, widget):
        """
        Met un tooltip d'aide sur un widget(bouton)
        """
        if self.eole_dico.helps['families'].has_key(family):
            text = self.eole_dico.helps['families'][family]
            tip = gtk.Tooltips()
            tip.set_tip(widget, text)
            tip.enable()
       
    def _check_has_enum(self, var):
        checks = self.eole_dico.variables[var].checks
        for check in checks:
            if check[0] == 'valid_enum':
                checkval = "True"
                for param in check[1]:
                    if param['name'] == 'checkval':
                        checkval =  param['value']
                liste = check[1][0]['value']
                return liste, checkval
        return None, None
        
    def _build_label(self,var):
        if 'obligatoire' in [check[0] for check in self.eole_dico.variables[var].checks]:
            texte = '<span font_desc="bold"> '+self.eole_dico.variables[var].description
        else:
            texte = '<span> '+self.eole_dico.variables[var].description
        if self.mode == 'expert':
            texte +=  '\n ( '+var+' )'
        texte += '</span>'
        if self.var_status.has_key(var):
            if self.var_status[var] != "":
                if self.mode != 'expert':
                    texte += "\n"
                # le message d'erreur démarre au 1er ":"
                texte += '<span foreground="#dd2222">%s</span>' % ':'.join(self.var_status[var].split(':')[1:])
        label = gtk.Label()
        label.set_markup(texte)
        label.set_justify(gtk.JUSTIFY_CENTER)
        #label.set_justify(gtk.JUSTIFY_LEFT)
        label.set_line_wrap(True)
        # largeur des libelles des variables
        label.set_size_request(530, -1) 
        label.show()
        view = gtk.EventBox()
        view.set_visible_window(False)
        view.add(label)
        view.show()
        return view

    def _build_table(self, family):
        """
        Construit la table selon les variables de la famille en cours
        @param family: famille en cours
        """
        cpt = 0
        self.stock = {}
        #on ajuste la taille de la table selon la famille
        self.table.resize(len(self.eole_dico.families[family]['vars']), 3)
        for var in self.eole_dico.families[family]['vars']:
            self.stock[var] = (None, None, None, None)
        for var in self.eole_dico.families[family]['vars']:
            # recherche des séparateurs
            if self.eole_dico.separators.has_key(var):
                separator = self.eole_dico.separators[var]
                # Création d'une boite avec le séparateur
                sep_box = gtk.HBox()
                sep_box.pack_start(gtk.HSeparator(), True,True)
                texte_sep = '<span foreground="#a0569a" font_desc="monospace bold 12">%s</span>' % separator
                label_sep = gtk.Label()
                label_sep.set_markup(texte_sep)
                sep_box.pack_start(label_sep, False, False)
                sep_box.pack_start(gtk.HSeparator(), True,True)
                sep_box.show_all()
                self.table.attach(sep_box, 0, 3, cpt, cpt+1, gtk.FILL|gtk.EXPAND, 0, 0, 4)
                cpt += 1
            # texte a afficher dans le label
            # pour les variables obligatoires, on affiche le texte en gras
            view=self._build_label(var)
            label_box = gtk.VBox()
            button_box = gtk.VBox()
            label_box.pack_start(view, False, False)
            label_box.show()
            # boite des champs de texte
            text_box = gtk.VBox()
            # boite des boutons +/-
            test_box = gtk.HBox()
            self.stock[var] = (text_box, label_box, button_box, test_box)
            values = self.eole_dico.variables[var].get_value()
            if values == []:
                values = ['']
            #self._add_text_field(None, var, values[0])
            for val in values:
                self._add_text_field(None, var, val)
            # bouton +/- pour ajouter/supprimer des valeurs (champs de type 'multi')
            test_box.pack_start(text_box, True, True)
            if self.eole_dico.variables[var].multi == True:
                button_add = gtk.Button('+')
                button_add.connect('clicked',self._add_text_field, var, None)
                button_add.unset_flags("GTK_CAN_FOCUS")
                button_add.set_size_request(15, 30) 
                button_add.show()
                button_del = gtk.Button('-')
                button_del.connect('clicked',self._remove_text_field, var)
                button_del.unset_flags("GTK_CAN_FOCUS")
                button_del.set_size_request(15, 30) 
                button_del.show()
                #button_hbox.pack_start(button_del, False, False)
                test_box.pack_start(button_del, False, False)
                #button_hbox.pack_start(button_add, False, False)
                test_box.pack_start(button_add, False, False)
            test_box.show()
            # on attache le label a la table
            self.table.attach(label_box, 0, 1, cpt, cpt+1, gtk.FILL|gtk.EXPAND, 0, 0, 0)
            # attache les champ de saisie
            self.table.attach(test_box, 1, 2, cpt, cpt+1, gtk.FILL|gtk.EXPAND, 0, 0, 2)
            button_hbox = gtk.HBox()
            
            # boutons "valeur précédente" et "valeur par défaut"
            button_prec = gtk.Button('Prec')
            button_def  = gtk.Button('Def')
            button_prec.set_size_request(40, 30)
            button_def.set_size_request(40, 30)
            button_prec.show()
            button_def.show()
            button_prec.connect('clicked',self._set_prec_value, var)
            button_def.connect('clicked',self._set_def_value, var)
            button_prec.unset_flags("GTK_CAN_FOCUS")
            button_def.unset_flags("GTK_CAN_FOCUS")
            button_hbox.pack_start(button_prec, False, False)
            button_hbox.pack_start(button_def, False, False)

            button_box.pack_start(button_hbox, False, False)
            button_box.show_all()
            self.table.attach(button_box, 2, 3, cpt, cpt+1, 0, 0, 3, 0)
            
            #ajoute l'aide (tooltip)
            self._set_label_tooltip(var, view, text_box)
            cpt += 1

        self._disable_edits()
        #self.table.set_homogeneous(True)
        self.table.show()
    
    def _add_text_field(self, widget, var, value = None):
        text_box = self.stock[var][0]
        label_box = self.stock[var][1]
        button_box = self.stock[var][2]
        enum = self._check_has_enum(var)
        # index dans la liste des valeurs
        index = len(text_box.get_children()) - 1
        if enum[0] == None:
            try:
                if value != None:
                    saisie_texte = value
                else:
                    if index == -1:
                        saisie_texte = self._get_saisie_text(var)[0]
                    else:
                        saisie_texte = text_box.get_children()[-1].get_text()
            except IndexError:
                saisie_texte = ''
            #champ de saisie
            saisie = gtk.Entry()
            saisie.set_size_request(200, 30)
            #texte a afficher dans le champ de saisie
            saisie.set_text(saisie_texte)
            saisie.connect('focus-in-event',self._callback_focus_in_edit, var)
        else:
            if enum[1] == "False":
                # on permet la saisie de valeur non présentes dans la liste
                saisie = gtk.combo_box_entry_new_text()
            else:
                saisie = gtk.combo_box_new_text()
            cpt_t = 0
            index_txt = 0
            if value != None:
                saisie_texte = value
            else:
                if index == -1:
                    saisie_texte = self._get_saisie_text(var)[0]
                else:
                    if len(text_box.get_children()) == 1:
                        input = text_box.get_children()[0]
                    else:
                        input = text_box.get_children()[-1]
                    model = input.get_model()
                    if enum[1] == "False":
                        saisie_texte = input.get_child().get_text()
                    else:
                        saisie_texte = input.get_active_text()
            if enum[1] == "False":
                for text in eval(enum[0]):
                    saisie.append_text(text)
                saisie.get_child().set_text(saisie_texte)
            else:
                for text in eval(enum[0]):
                    saisie.append_text(text)
                    if text == saisie_texte:
                        index_txt = cpt_t
                    cpt_t+=1
                saisie.set_active(index_txt)
            #if type(saisie) in (gtk.ComboBoxEntry, gtk.Combo):
            saisie.set_size_request(200, 30)
            #else:
            #    saisie.set_size_request(305, 35) 
            saisie.connect('changed', self._callback_combobox_changed, var)

        text_box.pack_start(saisie, False, False)
        # champ modifié => action
        if type(saisie) != gtk.ComboBox:
            saisie.connect('changed', self._callback_modified_textbox)
        
        if len(text_box.get_children()) != 1:
            # labels vides pour forcer l'alignement en haut (label et boutons)
            tmp = gtk.Label('')
            tmp.set_size_request(-1, 30)
            tmp.show()
            tmp2=gtk.Label('')
            tmp2.set_size_request(-1, 30)
            tmp2.show()
            label_box.pack_end(tmp, False, False)
            #label_box.pack_start(tmp, False, False)
            button_box.pack_end(tmp2, False, False)
            #button_box.pack_start(tmp2, False, False)
        saisie.show()
        text_box.show()

        # vérification des variables groupées
        if value == None:
            if self.eole_dico.groups.has_key(var) and len(self.stock[var][0].get_children()) != 1:
                for slave in self.eole_dico.groups[var]:
                    self._add_text_field(None, slave)
                

    def _remove_text_field(self, widget, var):
        text_box = self.stock[var][0]
        label_box = self.stock[var][1]
        button_box = self.stock[var][2]
        inputs = text_box.get_children()
        labels = label_box.get_children()
        buttons = button_box.get_children()
        if len(inputs) != 1:
            inputs[-1].destroy()
            labels[-1].destroy()
            buttons[-1].destroy()
            
        # vérification des variables groupées
        if self.eole_dico.groups.has_key(var):
            for slave in self.eole_dico.groups[var]:
                self._remove_text_field(None, slave)

    def _set_prec_value(self, widget, var):
        """
        On remplit le champ texte avec la valeur précédente
        """
        inputs = self.stock[var][0].get_children()
        val_prec = self.eole_dico.get_prec_value(var)
        for index in range(len(inputs)):
            textbox=inputs[index]
            try:
                val = val_prec[index]
            except:
                break
            if type(textbox) in (gtk.ComboBoxEntry, gtk.ComboBox):
                model = textbox.get_model()
                cur_iter = model.get_iter_root()
                while cur_iter:
                    if model.get_value(cur_iter,0) == val:
                        textbox.set_active_iter(cur_iter)
                        break
                    cur_iter=model.iter_next(cur_iter)
            else:
                textbox.set_text(val)

    def _set_def_value(self, widget, var):
        """
        On remplit le champ texte avec la valeur par défaut
        """
        inputs = self.stock[var][0].get_children()
        val_def = self.eole_dico.get_value(var, default=True)
        for ind in range(len(inputs)):
            textbox = inputs[ind]
            if len(self.eole_dico.variables[var].fills) > 0:
                self._callback_focus_in_edit(textbox, None, var, True)
            else:
                if len(val_def) > 1:
                    val = val_def[ind]
                else:
                    val = val_def[0]
                if type(textbox) in (gtk.ComboBoxEntry, gtk.ComboBox):
                    model = textbox.get_model()
                    cur_iter = model.get_iter_root()
                    while cur_iter:
                        if model.get_value(cur_iter,0) == val:
                            textbox.set_active_iter(cur_iter)
                            break
                        cur_iter=model.iter_next(cur_iter)
                else:
                    textbox.set_text(val)

    def _clear_table(self):
        """
        Detruit tous les widgets de la table
        """
        self.table.foreach(self._widget_destroy)
    
    def _widget_destroy(self, widget):
        """
        Detruit le widget passe en parametre
        """
        widget.destroy()

    def _untoggle_buttons(self, name):
        """
        ne met que le bouton passe en parametre en mode "toggled"
        """
        for i in self.button_stock.keys():
            if i != name:
                self.button_stock[i].set_active(False)
            else:
                self.button_stock[i].set_active(True)
 

    def _disable_buttons(self):
        """
        Active et desactive les boutons selon leur champ hidden
        """
        for family, value in self.eole_dico.families.items():
            if self.mode == 'normal' and family not in self.eole_dico.normal_families:
                continue
            if value['hidden'] == True:
                #self.button_stock[family].set_sensitive(False)
                self.button_stock[family].hide()
            else:
                #self.button_stock[family].set_sensitive(True)
                self.button_stock[family].show()

    def _disable_edits(self):
        """
        Active et desactive les labels et edits selon leur champ hidden
        """
        for key, value in self.stock.items():

            if self.eole_dico.variables[key].hidden == True:
                for input in value[0].get_children():
                    if self.hide_vars == False:
                        input.set_sensitive(False)
                    else:
                        input.hide()
                if self.hide_vars == False:
                    value[1].get_children()[0].set_sensitive(False)
                    value[2].get_children()[0].set_sensitive(False)
                    value[3].set_sensitive(False)
                else:
                    value[1].get_children()[0].hide()
                    value[2].get_children()[0].hide()
                    value[3].hide()
            else:
                for input in value[0].get_children():
                    if self.hide_vars == False:
                        input.set_sensitive(True)
                    else:
                        input.show()
                if self.hide_vars == False:
                    value[1].get_children()[0].set_sensitive(True)
                    value[2].get_children()[0].set_sensitive(True)
                    value[3].set_sensitive(True)
                else:
                    value[1].get_children()[0].show()
                    value[2].get_children()[0].show()
                    value[3].show()

    def _save_values(self, widget):
        """
        sauve les valeurs contenues dans les champ de saisie de la table
        """
        if self.edited_family != '':
            #sauvegarde les valeurs actuelles pour eventuellement les restaurer
            val_sauv = {}
            self._push_vals(val_sauv)
            # for i in self.stock.keys():
            erreurs = []
            for i in self.eole_dico.families[self.edited_family]['vars']:
                val = []
                # on récupère les valeurs dans les champs de la variable
                #gtk.Entry
                if type(self.stock[i][0].get_children()[0]) == gtk.Entry:
                    for input in self.stock[i][0].get_children():
                        chaine = input.get_text()
                        val.append(chaine.strip())
                #gtk.ComboBox
                else:
                    for input in self.stock[i][0].get_children():
                        model = input.get_model()
                        if type(input) == gtk.ComboBoxEntry:
                            saisie = input.get_child().get_text()
                        else:
                            saisie = input.get_active_text()
                        val.append(saisie)
                try:
                    self.eole_dico.set_value(i, val)
                    self.var_status[i] = ""
                except Exception, e:
                    erreurs.append(e.args[0])
                    # on tag la variable pour l'afficher en rouge
                    self.var_status[i] = e.args[0]

            if erreurs != []:
                self.button_stock[self.edited_family].set_image(self.build_led())
                if self.edited_family in self.valid_families:
                    self.valid_families.remove(self.edited_family)
                self._error_window("\n".join(erreurs))
                # mise à jour des libellés d'erreur
                for var, var_stat in self.var_status.items():
                    if var_stat != "":
                        label_box = self.stock[var][1]
                        # construction du libellé
                        label_box.get_children()[-1].destroy()
                        view=self._build_label(var)
                        label_box.pack_start(view, False, False)
                        label_box.show()
                # pas de refresh pour éviter de perdre les valeurs ajoutées
                # self._refresh_display()
                return False
            else:
                self.valid_families.append(self.edited_family)
                self.button_stock[self.edited_family].set_image(self.build_led(True))
                self._refresh_display()

            #active/desactive boutons de famille selon champ hidden
            self._disable_buttons()
            #active/desactive les labels/edits selon leur champ hidden
            self._disable_edits()
            #desactive le flag de sauvegarde
            self.save_flag = False
            self.modified_flag = False

        return True

    def _push_vals(self, dico):
        """
        sauvegarde les valeurs actuelles des variables de la famille actuelle
        """
        for var in self.stock.keys():
            dico[var] = {'val':self.eole_dico.variables[var].val}
            dico[var]['valprec'] = self.eole_dico.variables[var].valprec
            dico[var]['valdefault'] = self.eole_dico.variables[var].valdefault
            dico[var]['hidden'] = self.eole_dico.variables[var].hidden
            dico[var]['valeole'] = self.eole_dico.variables[var].valeole
                        
    def _pop_vals(self, dico):
        """
        restaure les valeurs des variables passees en parametre
        """
        for var in dico.keys():
            self.eole_dico.variables[var].val = dico[var]['val']
            self.eole_dico.variables[var].valprec = dico[var]['valprec']
            self.eole_dico.variables[var].valdefault = dico[var]['valdefault']
            self.eole_dico.variables[var].hidden = dico[var]['hidden']
            self.eole_dico.variables[var].valeole = dico[var]['valeole']

            
    def _check_groups(self):
        """
        verifie que les conditions de groupes sont respectees
        i.e que tous les slaves d'un master sont remplis
        """
        #key: nom du master
        hidden_vars = []
        for name, fam in self.eole_dico.families.items():
            if fam['hidden'] == True:
                hidden_vars.extend(fam['vars'])
        #value: liste des slaves
        for key, value in self.eole_dico.groups.items():
            if key not in hidden_vars:
                #si master a une valeur
                if self.eole_dico.variables[key].val != []:
                    lg_key = len(self.eole_dico.variables[key].val)
                    for slave in value:
                        lg_slave = len(self.eole_dico.variables[slave].val) 
                        if self.eole_dico.variables[slave].val == [] or lg_key != lg_slave:
                            text = 'Probleme de dependance entre '+key+' et '+slave+'. '+slave+' doit avoir une valeur.'
                            text += '\nOu '+slave+' doit avoir le meme nombre de valeurs que '+key
                            text += '\nEnregistrement non effectue'
                            self._error_window(text)
                            return False
        return True

    def _error_window(self, text, title='Erreur'):
        """
        affiche un message d'erreur
        """
        if type(text) != unicode:
            text = unicode(text,config.charset)
        text = text.encode("utf-8")
        MessageWindow (title, text, self.window, type="error")
       
    def _question_window(self, text, title='Question'):
        """
        Pose une question a l'utilisateur
        """
        return MessageWindow(title, text, self.window, type='yesno').getrc()
       
    def main(self):
        """
        lance l'interface
        """
        #construit et affiche les boutons a gauche
        self._build_buttons()
        #desactive les boutons dont le champ hidden est a True
        self._disable_buttons()
        self._disable_edits()
        #affiche la première page disponible
        for fam in self.eole_dico.family_order:
            if not self.eole_dico.families[fam]['hidden'] == True:
                if self.mode == 'normal' and fam not in self.eole_dico.normal_families:
                    continue
                self.button_stock[fam].set_active(True)
                break
        
        self.window.show()
        # self.window.maximize()

    def _change_mode(self, widget):
        """
        changement de mode
        """
        # fausse alerte !
        if widget.active :
            if self.mode == 'normal':
                return
        else:
            if self.mode == 'expert':
                return
        
        if self.modified_flag == True:
            # validation des changements ?
            reponse = self._question_window("Valider les modifications ?")
            if reponse == 1:
                if not self._save_values(None):
                    # erreur : on se remet dans l'ancien mode
                    if widget.active :
                        widget.set_active(False)
                    else:
                        widget.set_active(True)
                    return True
                
        # comme neuf
        self.modified_flag = False
        self.edited_family = ""
        
        # changement officiel
        if widget.active:
            self.mode = 'normal'
        else:
            self.mode = 'expert'
        self.main()


    def _change_affvar(self, widget):
        """
        changement de mode d'affichage des variables cachées
        """
        # fausse alerte !
        if widget.active :
            if self.hide_vars == False:
                return
        else:
            if self.hide_vars == True:
                return
        
        if self.modified_flag == True:
            # validation des changements ?
            reponse = self._question_window("Valider les modifications ?")
            if reponse == 1:
                if not self._save_values(None):
                    # erreur : on se remet dans l'ancien mode
                    if widget.active :
                        widget.set_active(False)
                    else:
                        widget.set_active(True)
                    return True
                
        self.modified_flag = False
        
        if widget.active:
            self.hide_vars = False
        else:
            self.hide_vars = True

        # réaffiche la page actuelle
        for fam in self.eole_dico.family_order:
            if fam == self.edited_family:
                self._callback_toggle_button(self.button_stock[fam], fam)
                break
        self.window.show()
        # self.window.maximize()


class MessageWindow:
    def getrc (self):
        return self.rc
    
    def __init__ (self, title, text, window, type="ok", default=None, custom_buttons=None, custom_icon=None):
        
        self.rc = None
        
        if type == 'ok':
            buttons = gtk.BUTTONS_OK
            style = gtk.MESSAGE_INFO
        elif type == 'warning':
            buttons = gtk.BUTTONS_OK
            style = gtk.MESSAGE_WARNING
        elif type == 'okcancel':
            buttons = gtk.BUTTONS_OK_CANCEL
            style = gtk.MESSAGE_WARNING
        elif type == 'yesno':
            buttons = gtk.BUTTONS_YES_NO
            style = gtk.MESSAGE_QUESTION
        elif type == 'error':
            buttons = gtk.BUTTONS_OK
            style = gtk.MESSAGE_ERROR
        elif type == 'custom':
            buttons = gtk.BUTTONS_NONE
            style = gtk.MESSAGE_QUESTION

        if custom_icon == "warning":
            style = gtk.MESSAGE_WARNING
        elif custom_icon == "question":
            style = gtk.MESSAGE_QUESTION
        elif custom_icon == "error":
            style = gtk.MESSAGE_ERROR
        elif custom_icon == "info":
            style = gtk.MESSAGE_INFO
        
        dialog = gtk.MessageDialog(window, 0, style, buttons, text)

        dialog.set_position (gtk.WIN_POS_CENTER)
        dialog.show_all ()

        rc = dialog.run()
        
        if rc == gtk.RESPONSE_OK or rc == gtk.RESPONSE_YES:
            self.rc = 1
        elif (rc == gtk.RESPONSE_CANCEL or rc == gtk.RESPONSE_NO
            or rc == gtk.RESPONSE_CLOSE):
            self.rc = 0
        elif rc == gtk.RESPONSE_DELETE_EVENT:
            self.rc = 0
        else:
            self.rc = rc
        dialog.destroy()


if __name__ == '__main__':
    # répertoire des dictionnaires par défaut
    dirs = ['/etc/eole/dicos/','/etc/eole/dicos/local/', '/etc/eole/dicos/variante/']
    # récupération de répertoires additionnels passés en argument
    option_parser = optparse.OptionParser("Usage: gen_config.py [-w] <repertoires> <fichier.eol>", version = "%prog")
    option_parser.add_option('-w', '--window-mode', dest = "window_mode", action = "store_true")
    (options, args) = option_parser.parse_args(sys.argv)
    config_file = None
    for arg in args:
        if arg != 'txt':
            if os.path.isfile(arg) and arg.endswith('.eol'):
                config_file = arg
            dirs.append(arg)

    gui = ConfigGui(glade_file,dirs,options=options,filename=config_file)
    gui.main()
    gtk.main()
