# -*- coding: UTF-8 -*-
###########################################################################
#
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# libsecure.py
#
# classes utilitaires pour lancement des services en https
#
###########################################################################

import os
from tempfile import mktemp
from creole.creolecat import instance
from creole.parsedico import parse_dico
from creole.config import cert_file, key_file

def gen_certif(certfile=None):
    """
    génère un certificat pour le server https
    """
    if certfile is None:
        certfile = cert_file
        keyfile = key_file
        requete_dir = os.path.dirname(certfile)
    else:
        requete_dir = os.path.dirname(certfile)
        basefile = os.path.splitext(os.path.basename(certfile))[0]
        keyfile = "%s/%s.key" % (requete_dir, basefile)
    if os.path.isfile(certfile):
        # certif déjà présent
        return True
    if not os.path.isdir(requete_dir):
        os.makedirs(requete_dir)

    conf_template = '/usr/share/creole/certif-eole.conf'
    conf_file = '/etc/sysconfig/eole/certif-eole.conf'
    if not os.path.isfile(conf_template):
        print "Fichier template de la configuration du certificat non trouvé :\n\n\t%s\n" % conf_template
        return False
    # instanciation du template
    instance(conf_template, None, conf_file, '/etc/eole/config.eol')
    if not os.path.isfile(conf_file):
        print 'Erreur de génération du fichier de configuration ssl'
        return False
    tmp_keyfile = mktemp()

    # génération certificat x509
    ssl_cmd = "openssl req -x509 -new -newkey rsa:1024 -days 1825 -config %s -extensions alt_names -keyout %s -out %s >/dev/null" % (conf_file, tmp_keyfile, certfile)
    res = os.system(ssl_cmd)
    if res != 0:
        print '\n! erreur lors de la génération du certificat ssl dans %s !\n' % certfile
        return False

    # génération de la clé publique
    key_cmd = "openssl rsa -in %s -passin pass:secret -out %s >/dev/null" % (tmp_keyfile, keyfile)
    res = os.system(key_cmd)
    if res != 0:
        print('\n! erreur lors de la génération de la clé ssl dans %s !\n' % keyfile)
    os.unlink(tmp_keyfile)
    # finalisation du fichier certificat
    dico = parse_dico()
    cert_data = open(certfile).read()
    key_data = open(keyfile).read()
    try:
        f_cert = open(certfile,'w')
        f_cert.write("SubjectAltName=%s.%s\n%s\n%s" % (dico['nom_machine'], 
            dico['nom_domaine_local'], cert_data.rstrip(), key_data.rstrip()))
        f_cert.close()
    except:
        print "\n! Erreur lors de l'écriture du fichier %s" % certfile
        return False

    return True


def gen_signed_certif(certfile=None):
    """
    génération de certificats auto-signés
    """
    fill = {}
    if certfile is None:
        fill['certfile'] = cert_file
        fill['keyfile'] = key_file
        requete_dir = os.path.dirname(cert_file)
    else:
        requete_dir = os.path.dirname(certfile)
        basefile = os.path.splitext(os.path.basename(certfile))[0]
        fill['certfile'] = certfile
        fill['keyfile'] = "%s/%s.key" % (requete_dir, basefile)
    if os.path.isfile(fill['certfile']):
        # certif déjà présent
        return True
    if not os.path.isdir(requete_dir):
        os.makedirs(requete_dir)
    if not os.path.isdir("/usr/share/doc/openvpn/examples/easy-rsa/2.0/"):
        print "repertoire %s manquant !" % "/usr/share/doc/openvpn/examples/easy-rsa/2.0/"
        return False

    dico = parse_dico()
    fill['server'] = "%s.%s" % (dico['nom_machine'], dico['nom_domaine_local'])
    if dico.has_key('numero_etab'):
        fill['city'] = dico['numero_etab']
    else:
        fill['city'] = dico['nom_domaine_local']
    if dico.has_key('nom_academie'):
        fill['acad'] = dico['nom_academie']
    else:
        fill['acad'] = dico['nom_domaine_academique']

    cmd = """#!/bin/bash
cd /usr/share/doc/openvpn/examples/easy-rsa/2.0/
source vars >/dev/null
# écrasement des variables supplémentaires
export KEY_COUNTRY="FR"
export KEY_PROVINCE="%(acad)s"
export KEY_CITY="%(city)s"
export KEY_ORG="Eole"
export KEY_EMAIL="eole@ac-dijon.fr"
./clean-all
./build-dh
./pkitool --initca 2>/dev/null
./pkitool --server %(server)s 2>/dev/null
cd keys
cp -f ca.crt /etc/ssl/certs
cp -f %(server)s.crt %(certfile)s
cp -f %(server)s.key %(keyfile)s
"""
    return os.system(cmd % fill) == 0

