# -*- coding: UTF-8 -*-

from creole import config, parsedico
from creole.error import FileNotFound, ConfigError
from creole.cfgparser import EoleDict
import os, time, sys, re, signal
import glob
from ConfigParser import ConfigParser
import subprocess


def patch():
    """
    applique les patchs
    """
    distrib_dir = config.distrib_dir
    patch_dir = config.patch_dir
    template_dir = config.templatedir
    var_dir = os.path.join(patch_dir,'variante')
    # sauvegarde du répertoire de départ
    cur_dir = os.getcwd()
    os.chdir(template_dir)
    # copie des templates originaux
    os.system('cp -f %s/* %s' % (distrib_dir, template_dir))
    # patches variante + locaux
    for directory in [var_dir, patch_dir]:
        patches = os.listdir(directory)
        for patch in patches:
            if patch.endswith('.patch'):
                os.system('patch -N -p1 < %s >> %s' % (os.path.join(directory, patch), config.logfile))
    os.chdir(cur_dir)

def update_defaults(update_file):
    """
    Mise à jour des valeurs par défaut si nécessaire
    """
    if os.path.isfile(update_file):
        store = ConfigParser()
        try:
            store.read(update_file)
        except:
            print "Fichier de mise à jour %s invalide" % update_file
            return
        need_save = False
        eole_dico = EoleDict()
        eole_dico.read_dir(config.eoledirs)
        eole_dico.load_values()
        for varname in store.sections():
            if eole_dico.variables.has_key(varname):
                var = eole_dico.variables[varname]
                old = store.get(varname, 'old')
                if old == '':
                    old = []
                else:
                    old = [old]
                if var.val == old and var.valprec == []:
                    try:
                        var.set_value(store.get(varname, 'new'))
                        print "La variable %s a été mise à jour" % varname
                        need_save = True
                    except:
                        print "Mise à jour de la varibale %s impossible" % varname
        if need_save:
            eole_dico.save_values()
        return

def get_services_actifs():
    """
    liste des services à activer
    (/etc/sysconfig/eole/services)
    """
    services_actifs = []
    serv_files = glob.glob('/etc/sysconfig/eole/services/*.srv')
    serv_files.sort()
    for serv_file in serv_files:
        for service in file(serv_file).read().split('\n'):
            service = service.strip()
            if service != '' and service not in services_actifs:
                services_actifs.append(service)
    return services_actifs

def get_services_in_templates():
    """
    liste des services définis dans les templates
    (/etc/eole/template)
    """
    services_in_templates = []
    serv_files = ['/etc/eole/template/services']
    serv_files.extend(glob.glob('/etc/eole/template/*.srv'))
    if os.path.isfile('/etc/sysconfig/eole/disabled.srv'):
        serv_files.append('/etc/sysconfig/eole/disabled.srv')
    for serv_file in serv_files:
        for service in file(serv_file).read().split('\n'):
            service = service.strip()
            if service != '' and not service.startswith('%'):
                if service not in services_in_templates:
                    services_in_templates.append(service)
    return services_in_templates

def activate_services():
    """
    active les services
    """
    services_actifs = get_services_actifs()
    services_dispos = get_services_in_templates()
    # services à désactiver
    for service in services_dispos:
        if service not in services_actifs:
            os.system('/usr/sbin/invoke-rc.d %s stop &>/dev/null' % service)
            os.system('/usr/sbin/update-rc.d -f %s remove >/dev/null' % service)
    # services à activer
    for service in services_actifs:
        os.system('/usr/sbin/update-rc.d %s defaults >/dev/null' % service)

def restart_services(ead=False):
    """
    redémarre les services actifs
    """
    print "** Redémarrage des services **"
    print ""
    for service in get_services_actifs():
        if ead and service in ['ead-server', 'ead-web', 'eole-sso']:
            continue
        elif service == 'mars-nwe':
            os.system('/etc/init.d/%s restart' % service)
        else:
            os.system('/usr/sbin/invoke-rc.d %s restart' % service)

def get_module_name(version=False):
    """
    renvoie le nom du module
    @param version: renvoie aussi la version si True
    """
    if not os.path.isfile(config.f_version):
        raise FileNotFound, '%s non trouvé' % config.f_version
    f_vers = open(config.f_version)
    content = f_vers.read().strip()
    f_vers.close()
    if content == '':
        raise ConfigError, '%s vide' % config.f_version
    if version:
        return content
    else:
        return content.split('-')[0]

def user_del():
    """
    suppression des comptes système inutiles
    """
    for user in ['games', 'irc', 'list', 'backup']:
        os.system("userdel %s &>/dev/null" % user)
    # suppression du bash pour nobody
    os.system("usermod -s /bin/false nobody &>/dev/null")

def test_vz(master=False):
    """
    master = True : on teste si c'est un serveur hôte
    master = False : on teste si c'est un serveur virtualisé
    """
    if master:
        package = 'eolevz'
        openvz = os.system('dpkg -l %s 2>/dev/null|grep -q "^ii"' % package)
        return openvz == 0
    else:
         if os.path.exists('/etc/sysconfig/eole/eolevz'):
             return True
         else:
             return False

def controle_kernel():
    """
    renvoie True si il faut rebooter (nouveau kernel)
    """
    if test_vz():
        return False
    regen_grub()
    grub_index = -1
    noyau_actuel = os.popen("uname -r", "r").readline().strip()
    try:
        grub_default = open("/boot/grub/default").read().split('\n')[0].strip()
        if grub_default != "default":
            current_grub_index = int(grub_default.replace('\x00',''))
        else:
            current_grub_index = -2
    except:
        print "\n!! erreur lors de la lecture de /boot/grub/default !!\n"
        return False

    boot_ok = 1
    if os.path.exists(config.REBOOT_FILE):
        # i.e. /var/run/reboot-required
        boot_ok = 0
    local_file = "%s_local" % config.KERNEL_FILE
    if os.path.exists(local_file):
        # l'utilisateur demande un noyau spécifique
        kernel_file = local_file
    else:
        kernel_file = config.KERNEL_FILE
    if os.path.exists(kernel_file):
        # on utilise le noyau spécifié
        new_index = -1
        dernier_noyau = open(kernel_file).read().strip()
        if test_vz(master=True):
            # hebergeur openvz
            if not dernier_noyau.endswith('vz'):
                dernier_noyau += 'vz'
                kfil = open(kernel_file, 'w')
                kfil.write('%s\n' % dernier_noyau)
                kfil.close()
        if dernier_noyau != noyau_actuel:
            boot_ok = 0
            grub_index = parse_menu_lst(dernier_noyau)
            if grub_index != -1:
                new_index = grub_index
        else:
            grub_index = parse_menu_lst(dernier_noyau)
            if current_grub_index != grub_index:
                # cas ou un nouveau noyau a été installé (mais pas le bon)
                new_index = grub_index
            if grub_index == -1:
                print "\n!! noyau non trouvé : %s !!\n" % dernier_noyau
        if new_index != -1:
            res = os.system('grub-set-default %s 2>/dev/null' % grub_index)
            if res != 0:
                print "\n!! erreur lors de la selection du noyau par défaut (index %s) !!\n" % grub_index
                return False
    else:
        dernier_noyau  = os.popen("ls -l /vmlinuz ", "r").readline().strip().split("boot/vmlinuz-")[1]
        if dernier_noyau != noyau_actuel:
            return True
    if boot_ok == 0:
        return True
    return False

def regen_grub():
    """ré-initialise les options menu.lst"""
    if os.path.exists('/boot/grub/menu.lst'):
        cmd = """sed -i "s/^timeout[ \t].*$/timeout 6/g" /boot/grub/menu.lst
        sed -i "s/^default[ \t+].*$/default saved/g" /boot/grub/menu.lst
        sed -i "s/^hiddenmenu/#hiddenmenu/g" /boot/grub/menu.lst
        sed -i "s/^# savedefault=false/# savedefault=true/g" /boot/grub/menu.lst
        /sbin/update-grub -y 2>/dev/null"""
        os.system(cmd)

def parse_menu_lst(dernier_noyau):
    """ récupération de l'index grub du noyau """
    menu = open('/boot/grub/menu.lst','r')
    express = "^title[ \t]+(.*)$"

    lines = menu.readlines()
    menu.close()
    kernels = []
    for i in lines:
        match = re.search(express, i)
        if match:
            kernels.append(match.group(1))

    for i in range(len(kernels)):
        if dernier_noyau in kernels[i] and 'kernel' in kernels[i] and not '(recovery mode)' in kernels[i]:
            return i
    return -1

def hold_kernel_package():
    """
    Met en statut hold le kernel Ubuntu pour éviter les MAJ
    """
    # FIXME : cette fonction python n'est jamais appelée !
    kerns = ['linux-server', 'linux-image-server', 'linux-image-2.6.22-15-server']
    for kernel in kerns:
        pop = subprocess.Popen(args=["dpkg --get-selections | grep %s" % kernel], stdout=subprocess.PIPE, shell=True)
        status = pop.stdout.read().split()
        if status and status[1] == 'install':
            rcode = subprocess.call(args=["echo %s hold | dpkg --set-selections" % kernel], shell=True)

def get_old_kernels():
    """
    Récupération de la liste des anciens noyaux (2.6.20 & 2.6.22) encore présents
    """
    # Merci de faire TRES ATTENTION si vous modifiez cette fonction !
    old_kernels = []
    for kernel in ['linux-image-2.6.20-*', 'linux-image-2.6.22-*', 'linux-image-2.6.24-21-*']:
        cmd = """COLUMNS=180 dpkg -l %s 2>/dev/null|grep -E "^ii|^hi"|awk -F " " '{print $2}'""" % kernel
        old_kernels.extend(os.popen(cmd).read().splitlines())
    return old_kernels

def remove_package():
    """
    Suppresion des paquets demandés par la liste /etc/sysconfig/eole/remove
    """
    if os.path.exists('/etc/eole/version') and os.path.exists(config.KERNEL_FILE):
        kernel_file = config.KERNEL_FILE
        module = open("/etc/eole/version").readline().strip().split("-")[0]
        current_kernel = open(kernel_file).readline().strip()
        if module in ['amonecole', 'eolevz'] and not current_kernel.endswith('vz'):
            kernel = open(kernel_file).readline().strip()
            kernel += 'vz'
            kfil = open(kernel_file, 'w')
            kfil.write('%s\n' % kernel)
            kfil.close()
    runkernel = os.popen('uname -r').read().strip()
    ficlog = open('/var/log/remove-package-eole.log','w')
    if not os.path.isfile('/etc/sysconfig/eole/remove'):
        print "Fichier /etc/sysconfig/eole/remove non trouvé"
        return False
    if test_vz():
        # openvz : on estime qu'on est sur le bon noyau
        fickernel = runkernel
    elif not os.path.isfile(config.KERNEL_FILE):
        print "Fichier %s non trouvé" % config.KERNEL_FILE
        return False
    else:
        fickernel = open(config.KERNEL_FILE).read().strip()
    remove = open('/etc/sysconfig/eole/remove').read().splitlines()
    remove.extend(get_old_kernels())
    for deb in remove:
        deb = deb.strip()
        process = subprocess.Popen(args=["dpkg", "-l", deb.strip()], stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False)
        code = process.wait()
        if code != 0:
            # paquet non installé
            continue
        dpkg_info = process.stdout.read()
        pkg_status = dpkg_info.strip().split('\n')[-1].split()
        if pkg_status[0].lower() in ['pn', 'un']:
            # ancien paquet purgé (status = pn) ou desinstallé (status = un)
            continue
        # on vérifie que le status est bien "installé"
        if deb.startswith("linux-image"):
            if deb.replace("linux-image-", '') == runkernel:
                # on ne peut pas supprimer le noyau courant
                continue
            if deb.replace("linux-image-", '') == fickernel:
                # on ne doit pas supprimer notre noyau officiel
                continue
        ficlog.write("===================== %s ====================="%time.ctime())
        print "Effacement du paquet %s ... " % deb.strip()
        pop = subprocess.Popen(args=["apt-get remove -q --purge --force-yes --assume-yes %s" % deb], stdout=subprocess.PIPE, shell=True)
        output = pop.stdout.read()
        ficlog.write(output)
    ficlog.close()
    return True

def purge_rc():
    """
    Purge des paquets "rc"
    """
    cmd = """COLUMNS=180 dpkg -l|grep "^rc"|awk -F " " '{print $2}'"""
    rcs = os.popen(cmd).read().splitlines()
    for pak in rcs:
        os.system("dpkg -P %s >/dev/null" % pak)

def install_package():
    """
    Ajout des paquets demandés dans /etc/sysconfig/eole/install
    """
    # ATTENTION : bien réflechir avant d'utiliser cette fonction !
    ficlog = open('/var/log/remove-package-eole.log','w')

    if not os.path.isfile('/etc/sysconfig/eole/install'):
        print "Fichier /etc/sysconfig/eole/install non trouvé"
        return False

    for deb in open('/etc/sysconfig/eole/install'):

        code = subprocess.call(args=["/usr/bin/dpkg", "-l", deb.strip()], stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False)

        if code :
            ficlog.write("===================== %s ====================="%time.ctime())
            print "Install du paquet %s ..." % deb.strip()
            pop = subprocess.Popen(args=["apt-get install -q --force-yes --assume-yes %s" % deb], stdout=subprocess.PIPE, shell=True)
            output = pop.stdout.read()
            ficlog.write(output)

    ficlog.close()

def update_mac():
    """
    mise à jour des adresses MAC dans les fichiers ethX.dev
    """
    mac_str = 'MAC='
    dmac_str = 'DEVICE_MAC='
    cmd = os.popen('/sbin/ifconfig -a | grep HWaddr')
    macs = cmd.read().split('\n')
    cmd.close()
    net_files = glob.glob('/etc/sysconfig/network/eth*.dev')
    for net_file in net_files:
        eth = os.path.basename(net_file).split('.')[0]
        for mac in macs:
            if mac.startswith(eth):
                print "Sauvegarde de l'adresse MAC pour %s\n" % eth
                conf_file = open(net_file,"r")
                buffer_in = conf_file.readlines()
                conf_file.close()
                buffer_out = ""
                for line in buffer_in:
                    if line.startswith(mac_str):
                        buffer_out += "%s'%s'\n" % (mac_str, mac.split()[4])
                    elif line.startswith(dmac_str):
                        buffer_out += "%s'%s'\n" % (dmac_str, mac.split()[4])
                    else:
                        buffer_out += line
                conf_file = open(net_file,"w")
                conf_file.write(buffer_out)
                conf_file.close()
                break

def zephir(etat, msg, type_proc):
    """ gestion des messages Zephir """
    # selon l'etat donné, on indique l'état de la tache
    etat_zeph = None
    if etat.upper().startswith("INIT"):
        etat_zeph = -1
    elif etat.upper().startswith("FIN"):
        etat_zeph = 0
    elif etat.upper().startswith('ERR'):
        etat_zeph = 1
    elif etat.upper().startswith('MSG'):
        etat_zeph = -2
    # log local si msg ou erreur
    if (len(msg) > 0) or (etat.upper() == "ERR"):
        log(etat, msg, type_proc)
    # log sur zephir si disponible
    if etat_zeph is not None and config.zephir_actif:
        config.log(type_proc, etat_zeph, msg, str(time.ctime()))

def log(etat, msg, type_proc):
    """
    effectue un log local et éventuellement sur zephir
    """
    if etat == "ERR":
        sys.stderr.write("\n%s : Erreur => %s\n" % (type_proc, msg))
    # log local
    os.system("""/usr/bin/logger -t \"zephir\" -p local2.err  \"%s => %s : %s \" """ % (type_proc, etat, msg))

def init_proc(type_proc):
    """
    initialisation d'une procédure (log démarrage + bcage éventuel)
    """
    if  verify_lock(type_proc):
        return True
    else:
        return False

#def end_proc(etat,msg,type_proc):
#    """
#    loggue la fin d'une procédure
#    """
#     log(etat,msg,type_proc )

def verify_lock(name):
    """
    vérifie le bloquage ou non d'une  procédure
    """
    LOCK_FILE = "/usr/share/zephir" + os.sep + "zephir_locks"
    if name == "":
        return True
    from zephir.lib_zephir import zephir_path
    try:
        from zephir.lib_zephir import config, zephir, convert
        locks = convert(zephir.serveurs.get_locks(config.id_serveur))
        if locks[0] == 0:
            # erreur sur zephir, on ignore cette phase
            raise Exception
        locks = [lock[0] for lock in locks[1]]
    except Exception, mess:
        # pas de message d'erreur si le serveur n'est pas enregistré
        if os.path.isfile(os.path.join(zephir_path, "zephir_conf", "zephir_conf.py")):
            # on ne bloque pas si l'appel à zephir échoue
            print "erreur d'accès à zephir : %s" % str(mess)
        # on regarde le denier état
        if os.path.exists(LOCK_FILE):
            file_lock = file(LOCK_FILE)
            locks = file_lock.read().split('\n')
            file_lock.close()
            # on bloque si interdit
            if name in locks:
                return False
    else:
        # mise en place du fichier de droits
        content = "\n".join(locks)
        try:
            file_lock = file(LOCK_FILE, "w")
            file_lock.write(content)
            file_lock.close()
        except:
            print "impossible de mettre à jour %s (droits insuffisants)" % LOCK_FILE
            pass
        # retour du code
        if name in locks:
            return False
    return True

def register_ead(dico):
    """
    Enregistrement de l'EAD local
    """
    if dico['adresse_ip_eth0'] == '':
        return
    from os.path import join
    sys.path.insert(1, '/usr/share')
    try:
        from ead2.frontend.web.lib.tools import servers
        from ead2.backend.lib.eadserver import EadKeyParser
        from ead2.backend.lib.eadserver import create_magicnb
        from ead2.backend.config.config import CONFIG_DIR, FRONTEND_KEYS_FILE
        from ead2.config.config import BACKEND_LISTEN_PORT
    except:
        return
    ead = EadKeyParser()
    ead.parse_file(join(CONFIG_DIR, FRONTEND_KEYS_FILE))
    if ead.get_key_dict() != {}:
        # des frontend sont déjà enregistrés
        return
    key = create_magicnb('admin','admin')
    dic = {dico['adresse_ip_eth0'] : key}
    # ecriture clé ead-serveur
    ead.write_file(join(CONFIG_DIR, FRONTEND_KEYS_FILE), dic)
    # ecriture clé ead-web
    servers().add_server('https://%s'%dico['adresse_ip_eth0'], str(BACKEND_LISTEN_PORT), dico['nom_machine'], str(key))

def register_module(ask=False):
    """
    Enregistrement Eole
    """
    if test_vz():
        # les serveurs virtuels sont exemptés d'enregistrement
        return
    tool_dir = '/usr/share/register'
    if not os.path.isdir(tool_dir):
        # eole-register pas installé ?
        return
    if not os.path.isfile(os.path.join(tool_dir,'.registerok')):
        # pas d'enregistrement préalable
        if not ask:
            return
        print
        print " http://eole.ac-dijon.fr/materiel/ vous permet de consulter une base des matériels compatibles"
        print "Pour enrichir cette base, acceptez-vous l'envoi de la description matérielle de ce serveur (oui/non) ?"
        rep = raw_input("[oui] : ")
        if rep.lower() in ['non', 'n']:
            return
        file(os.path.join(tool_dir, '.registerok'), 'w').close()

    sys.path.append('/usr/share/')
    from register import register
    os.chdir(tool_dir)
    register.initialize()

def proper_exit(*args):
    """callback à appeler lors d'un ctrl-c"""
    print("Abandon utilisateur")
    os._exit(1)

def catch_signal(exit_func=None):
    """gestion du ctrl-c"""
    if not exit_func:
        signal.signal(signal.SIGINT, proper_exit)
    else:
        signal.signal(signal.SIGINT, exit_func)

