# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Librairie de fonctions pour creole2
#
###########################################################################

from creole.fonctionseole import get_module_name
from os.path import join, basename
from os import system
import commands
import random

# fichiers ead2
TEMP_DIR = '/usr/share/ead2/backend/tmp'
AUTO_FILENAME = join(TEMP_DIR, 'cron.txt')
DIFF_FILENAME = join(TEMP_DIR, 'maj-diff.txt')
SCRIPT_DIR = '/usr/share/ead2/backend/bin/'
MAJ_FILENAME = join(SCRIPT_DIR, 'lance-maj.sh')


################################################
# Gestion de la mise à jour automatique (cron) #
################################################

def write_auto(enable, day=None, hour=None, minute=None):
    """ ecrit les informations du gestionnaire de mise a jour
        dans le fichier de config de l'ead """
    fic = file(AUTO_FILENAME, 'w')
    if enable:
        traduct = ['Dimanche', 'Lundi', 'Mardi', 'Mercredi', 
                   'Jeudi', 'Vendredi', 'Samedi', 'Dimanche']
        fic.write("Une mise à jour s'effectuera tous les %s à %0.2d:%0.2d" % (traduct[day], int(hour), int(minute)))
    else:
        fic.write("")
    fic.close()

def enable_maj_auto():
    """
    Programmation automatique d'une mise à jour hebdomadaire
    Elle est lancée via la crontab de l'utilisateur <module>
    """
    day = random.randint(0, 7)
    hour = random.randint(0, 6)
    minute = random.randint(0, 60)
    write_auto(True, day, hour, minute)
    modulename = get_module_name()
    cmd = '/bin/echo "%s %s * * %s %s -ead >/dev/null 2>&1" | /usr/bin/crontab -u %s -' % \
          (minute, hour, day, MAJ_FILENAME, modulename)
    ret, mess = commands.getstatusoutput(cmd)
    if ret:
        return False
    return True

def disable_maj_auto():
    """ 
    Déprogramme la mise à jour hebdomadaire automatique
    La crontab de l'utilisateur <module> est écrasée
    """
    write_auto(False)
    modulename = get_module_name()
    cmd = "/usr/bin/crontab -u %s -r" % modulename
    ret, mess = commands.getstatusoutput(cmd)
    if ret:
        return False
    return True

################################################
#  Gestion de la mise à jour programmée (at)   #
################################################

def write_diff(enable, heure=None):
    """ ecrit les informations du gestionnaire de mise a jour
        dans le fichier de config de l'ead """
    fic = file(DIFF_FILENAME, 'w')
    if enable:
        fic.write('Une mise à jour a été programmée le %s\n' % heure)
    else:
        fic.write("")
    fic.close()

def cancel_maj_differee():
    """ deprogramme une mise a jour differee """
    cmd = """for i in `grep -l "%s" /var/spool/cron/atjobs/* 2>/dev/null`; do rm -f $i ;  done;"""
    system(cmd % basename(MAJ_FILENAME))
    write_diff(False)
    return True

def prog_maj_differee(heure, options="-ead"):
    """ 
    Programmation une mise à jour différée de quelques heures
    Elle est lancée via la commande at pour l'utilisateur root
    """
    # suppression des éventuelles autres maj différées
    cancel_maj_differee()
    cmd = 'echo "%s %s\nrm -f %s" | at now + "%s hours"' % (MAJ_FILENAME, options, DIFF_FILENAME, heure)
    ret, mess = commands.getstatusoutput(cmd)
    if ret:
        return False
    parsed = mess.splitlines()[1].split()
    write_diff(True , ' '.join(parsed[3:7]))
    return True

